@@ -1,4 +1,3 @@
-/* $XFree86: xc/extras/Mesa/src/X86/assyntax.h,v 1.16 2003/11/06 18:37:53 tsi Exp $ */
 
 #ifndef __ASSYNTAX_H__
 #define __ASSYNTAX_H__
@@ -79,11 +78,9 @@
 #endif
 
 #if (defined(__STDC__) && !defined(UNIXCPP)) || (defined (sun) && defined (i386) && defined (SVR4) && defined (__STDC__) && !defined (__GNUC__))
-#define CONCAT(x, y)		x ## y
-#define CONCAT3(x, y, z)	x ## y ## z
+#define CONCAT(x, y)	x ## y
 #else
-#define CONCAT(x, y)		x/**/y
-#define CONCAT3(x, y, z)	x/**/y/**/z
+#define CONCAT(x, y)	x/**/y
 #endif
 
 #ifdef ACK_ASSEMBLER
@@ -865,8 +862,7 @@
 #define P_XMM6		xmm6
 #define P_XMM7		xmm7
 
-#define CONCAT(x, y)		x ## y
-#define CONCAT3(x, y, z)	x ## y ## z
+#define CONCAT(x, y)	x ## y
 
 #if defined(NASM_ASSEMBLER)
 
@@ -1692,10 +1688,8 @@ SECTION _DATA public align=16 class=DATA use32 flat
 /* Added by BrianP for FreeBSD (per David Dawes) */
 #if !defined(NASM_ASSEMBLER) && !defined(MASM_ASSEMBLER) && !defined(__bsdi__)
 #define LLBL(a)		CONCAT(.L,a)
-#define LLBL2(a,b)	CONCAT3(.L,a,b)
 #else
 #define LLBL(a)		a
-#define LLBL2(a,b)	CONCAT(a,b)
 #endif
 
 
