@@ -24,7 +24,7 @@
  * Authors:
  *    Keith Whitwell <keith@tungstengraphics.com>
  */
-/* $XFree86: xc/lib/GL/mesa/src/drv/mga/mgastate.c,v 1.15 2003/12/02 13:02:38 alanh Exp $ */
+/* $XFree86: xc/lib/GL/mesa/src/drv/mga/mgastate.c,v 1.14 2003/09/28 20:15:17 alanh Exp $ */
 
 
 #include "mtypes.h"
@@ -48,8 +48,13 @@
 #include "tnl/t_pipeline.h"
 #include "swrast_setup/swrast_setup.h"
 
+
 static void updateSpecularLighting( GLcontext *ctx );
 
+
+/* Some outstanding problems with accelerating logic ops...
+ */
+#if defined(ACCEL_ROP)
 static const GLuint mgarop_NoBLK[16] = {
    DC_atype_rpl  | 0x00000000, DC_atype_rstr | 0x00080000,
    DC_atype_rstr | 0x00040000, DC_atype_rpl  | 0x000c0000,
@@ -60,6 +65,8 @@ static const GLuint mgarop_NoBLK[16] = {
    DC_atype_rpl  | 0x00030000, DC_atype_rstr | 0x000b0000,
    DC_atype_rstr | 0x00070000, DC_atype_rpl  | 0x000f0000
 };
+#endif
+
 
 /* =============================================================
  * Alpha blending
@@ -104,32 +111,27 @@ static void mgaDDAlphaFunc(GLcontext *ctx, GLenum func, GLfloat ref)
       break;
    }
 
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+   FLUSH_BATCH( mmesa );
    mmesa->hw.alpha_func = a | MGA_FIELD( AC_atref, refByte );
-}
-
-static void updateBlendLogicOp(GLcontext *ctx)
-{
-   mgaContextPtr mmesa = MGA_CONTEXT(ctx);
-
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
-
-   mmesa->hw.blend_func_enable =
-      (ctx->Color.BlendEnabled && !ctx->Color._LogicOpEnabled) ? ~0 : 0;
-
-   FALLBACK( ctx, MGA_FALLBACK_BLEND,
-             ctx->Color.BlendEnabled && !ctx->Color._LogicOpEnabled &&
-             mmesa->hw.blend_func == (AC_src_src_alpha_sat | AC_dst_zero) );
+   mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 }
 
 static void mgaDDBlendEquation(GLcontext *ctx, GLenum mode)
 {
-   updateBlendLogicOp( ctx );
+   FLUSH_BATCH( MGA_CONTEXT(ctx) );
+
+   /* BlendEquation sets ColorLogicOpEnabled in an unexpected 
+    * manner.  
+    */
+   FALLBACK( ctx, MGA_FALLBACK_LOGICOP,
+	     (ctx->Color.ColorLogicOpEnabled && 
+	      ctx->Color.LogicOp != GL_COPY));
 }
 
 static void mgaDDBlendFunc(GLcontext *ctx, GLenum sfactor, GLenum dfactor)
 {
    mgaContextPtr mmesa = MGA_CONTEXT(ctx);
+   mgaScreenPrivate *mgaScreen = mmesa->mgaScreen;
    GLuint   src;
    GLuint   dst;
 
@@ -148,11 +150,11 @@ static void mgaDDBlendFunc(GLcontext *ctx, GLenum sfactor, GLenum dfactor)
    case GL_ONE_MINUS_SRC_ALPHA:
       src = AC_src_om_src_alpha; break;
    case GL_DST_ALPHA:
-      src = (ctx->Visual.alphaBits > 0)
+      src = (mgaScreen->cpp == 4) 
 	  ? AC_src_dst_alpha : AC_src_one;
       break;
    case GL_ONE_MINUS_DST_ALPHA:
-      src = (ctx->Visual.alphaBits > 0)
+      src = (mgaScreen->cpp == 4)
 	  ? AC_src_om_dst_alpha : AC_src_zero;
       break;
    case GL_SRC_ALPHA_SATURATE:
@@ -176,21 +178,18 @@ static void mgaDDBlendFunc(GLcontext *ctx, GLenum sfactor, GLenum dfactor)
    case GL_ONE_MINUS_SRC_COLOR:
       dst = AC_dst_om_src_color; break;
    case GL_DST_ALPHA:
-      dst = (ctx->Visual.alphaBits > 0)
+      dst = (mgaScreen->cpp == 4)
 	  ? AC_dst_dst_alpha : AC_dst_one;
       break;
    case GL_ONE_MINUS_DST_ALPHA:
-      dst = (ctx->Visual.alphaBits > 0)
+      dst = (mgaScreen->cpp == 4)
 	  ? AC_dst_om_dst_alpha : AC_dst_zero;
       break;
    }
 
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+   FLUSH_BATCH( mmesa );
    mmesa->hw.blend_func = (src | dst);
-
-   FALLBACK( ctx, MGA_FALLBACK_BLEND,
-             ctx->Color.BlendEnabled && !ctx->Color._LogicOpEnabled &&
-             mmesa->hw.blend_func == (AC_src_src_alpha_sat | AC_dst_zero) );
+   mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 }
 
 static void mgaDDBlendFuncSeparate( GLcontext *ctx, GLenum sfactorRGB,
@@ -200,6 +199,7 @@ static void mgaDDBlendFuncSeparate( GLcontext *ctx, GLenum sfactorRGB,
    mgaDDBlendFunc( ctx, sfactorRGB, dfactorRGB );
 }
 
+
 /* =============================================================
  * Depth testing
  */
@@ -233,9 +233,10 @@ static void mgaDDDepthFunc(GLcontext *ctx, GLenum func)
       zmode = 0; break;
    }
 
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+   FLUSH_BATCH( mmesa );
    mmesa->hw.zmode &= DC_zmode_MASK;
    mmesa->hw.zmode |= zmode;
+   mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 }
 
 static void mgaDDDepthMask(GLcontext *ctx, GLboolean flag)
@@ -243,9 +244,10 @@ static void mgaDDDepthMask(GLcontext *ctx, GLboolean flag)
    mgaContextPtr mmesa = MGA_CONTEXT( ctx );
 
 
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+   FLUSH_BATCH( mmesa );
    mmesa->hw.zmode &= DC_atype_MASK;
    mmesa->hw.zmode |= (flag) ? DC_atype_zi : DC_atype_i;
+   mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 }
 
 
@@ -341,7 +343,8 @@ static void mgaDDCullFaceFrontFace(GLcontext *ctx, GLenum unused)
 {
    mgaContextPtr mmesa = MGA_CONTEXT(ctx);
 
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+
+   FLUSH_BATCH( mmesa );
    if (ctx->Polygon.CullFlag && 
        ctx->Polygon.CullFaceMode != GL_FRONT_AND_BACK) 
    {
@@ -360,6 +363,8 @@ static void mgaDDCullFaceFrontFace(GLcontext *ctx, GLenum unused)
       mmesa->hw.cull = _CULL_DISABLE;
       mmesa->hw.cull_dualtex = _CULL_DISABLE;
    }
+
+   mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 }
 
 
@@ -484,11 +489,14 @@ static void updateSpecularLighting( GLcontext *ctx )
    mgaContextPtr mmesa = MGA_CONTEXT(ctx);
    unsigned int specen;
 
-   specen = (ctx->_TriangleCaps & DD_SEPARATE_SPECULAR) ? TMC_specen_enable : 0;
+   specen = (ctx->Light.Model.ColorControl == GL_SEPARATE_SPECULAR_COLOR &&
+	     ctx->Light.Enabled) ? TMC_specen_enable : 0;
 
    if ( specen != mmesa->hw.specen ) {
       mmesa->hw.specen = specen;
       mmesa->dirty |= MGA_UPLOAD_TEX0 | MGA_UPLOAD_TEX1;
+      
+      mgaChooseVertexState( ctx );
    }
 }
 
@@ -508,6 +516,14 @@ static void mgaDDLightModelfv(GLcontext *ctx, GLenum pname,
 }
 
 
+static void mgaDDShadeModel(GLcontext *ctx, GLenum mode)
+{
+   /* FIXME: This used to FLUSH_BATCH and set MGA_NEW_TEXTURE in new_state,
+    * FIXME: so I'm not sure what to do here now.
+    */
+}
+
+
 /* =============================================================
  * Stencil
  */
@@ -520,7 +536,7 @@ static void mgaDDStencilFunc(GLcontext *ctx, GLenum func, GLint ref,
    GLuint  stencil;
    GLuint  stencilctl;
 
-   stencil = MGA_FIELD( S_sref, ref ) | MGA_FIELD( S_smsk, mask );
+   stencil = (ref << S_sref_SHIFT) | (mask << S_smsk_SHIFT);
    switch (func)
    {
    case GL_NEVER:
@@ -550,20 +566,22 @@ static void mgaDDStencilFunc(GLcontext *ctx, GLenum func, GLint ref,
       break;
    }
 
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+   FLUSH_BATCH( mmesa );
    mmesa->hw.stencil &= (S_sref_MASK & S_smsk_MASK);
    mmesa->hw.stencil |= stencil;
    mmesa->hw.stencilctl &= SC_smode_MASK;
    mmesa->hw.stencilctl |= stencilctl;
+   mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 }
 
 static void mgaDDStencilMask(GLcontext *ctx, GLuint mask)
 {
    mgaContextPtr mmesa = MGA_CONTEXT(ctx);
 
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+   FLUSH_BATCH( mmesa );
    mmesa->hw.stencil &= S_swtmsk_MASK;
-   mmesa->hw.stencil |= MGA_FIELD( S_swtmsk, mask );
+   mmesa->hw.stencil |= (mask << S_swtmsk_SHIFT);
+   mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 }
 
 static void mgaDDStencilOp(GLcontext *ctx, GLenum fail, GLenum zfail,
@@ -663,10 +681,11 @@ static void mgaDDStencilOp(GLcontext *ctx, GLenum fail, GLenum zfail,
       break;
    }
 
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+   FLUSH_BATCH( mmesa );
    mmesa->hw.stencilctl &= (SC_sfailop_MASK & SC_szfailop_MASK 
 			    & SC_szpassop_MASK);
    mmesa->hw.stencilctl |= stencilctl;
+   mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 }
 
 
@@ -737,8 +756,14 @@ static void mgaDDLogicOp( GLcontext *ctx, GLenum opcode )
 {
    mgaContextPtr mmesa = MGA_CONTEXT( ctx );
 
-   MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+   FLUSH_BATCH( mmesa );
+#if defined(ACCEL_ROP)
    mmesa->hw.rop = mgarop_NoBLK[ opcode & 0x0f ];
+   mmesa->dirty |= MGA_UPLOAD_CONTEXT;
+#else
+   FALLBACK( ctx, MGA_FALLBACK_LOGICOP,
+	     (ctx->Color.ColorLogicOpEnabled && opcode != GL_COPY) );
+#endif
 }
 
 
@@ -868,24 +893,22 @@ static void mgaDDEnable(GLcontext *ctx, GLenum cap, GLboolean state)
    mgaContextPtr mmesa = MGA_CONTEXT( ctx );
 
    switch(cap) {
-   case GL_DITHER:
-      MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
-      if (!ctx->Color.DitherFlag)
-	 mmesa->setup.maccess |= MA_nodither_enable;
-      else
-	 mmesa->setup.maccess &= ~MA_nodither_enable;
-      break;
-   case GL_LIGHTING:
-   case GL_COLOR_SUM_EXT:
-      FLUSH_BATCH( mmesa );
-      updateSpecularLighting( ctx );
-      break;
    case GL_ALPHA_TEST:
-      MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+      FLUSH_BATCH( mmesa );
       mmesa->hw.alpha_func_enable = (state) ? ~0 : 0;
       break;
+   case GL_BLEND:
+      FLUSH_BATCH( mmesa );
+      mmesa->hw.blend_func_enable = (state) ? ~0 : 0;
+
+      /* For some reason enable(GL_BLEND) affects ColorLogicOpEnabled.
+       */
+      FALLBACK( ctx, MGA_FALLBACK_LOGICOP,
+		(ctx->Color.ColorLogicOpEnabled && 
+		 ctx->Color.LogicOp != GL_COPY));
+      break;
    case GL_DEPTH_TEST:
-      MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+      FLUSH_BATCH( mmesa );
       FALLBACK (ctx, MGA_FALLBACK_DEPTH,
 		ctx->Depth.Func == GL_NEVER && ctx->Depth.Test);
       break;
@@ -902,6 +925,8 @@ static void mgaDDEnable(GLcontext *ctx, GLenum cap, GLboolean state)
 	 mmesa->setup.maccess |= MA_fogen_enable;
       else
 	 mmesa->setup.maccess &= ~MA_fogen_enable;
+      
+      mgaChooseVertexState( ctx );
       break;
    case GL_CULL_FACE:
       mgaDDCullFaceFrontFace( ctx, 0 );
@@ -912,20 +937,22 @@ static void mgaDDEnable(GLcontext *ctx, GLenum cap, GLboolean state)
       break;
    case GL_POLYGON_STIPPLE:
       if (mmesa->haveHwStipple && mmesa->raster_primitive == GL_TRIANGLES) {
-	 MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+	 FLUSH_BATCH(mmesa);
+	 mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 	 mmesa->setup.dwgctl &= ~(0xf<<20);
 	 if (state)
 	    mmesa->setup.dwgctl |= mmesa->poly_stipple;
       }
       break;
-
-   case GL_BLEND:
    case GL_COLOR_LOGIC_OP:
-      updateBlendLogicOp( ctx );
+      FLUSH_BATCH( mmesa );
+#if !defined(ACCEL_ROP)
+      FALLBACK( ctx, MGA_FALLBACK_LOGICOP, 
+		(state && ctx->Color.LogicOp != GL_COPY));
+#endif
       break;
-
    case GL_STENCIL_TEST:
-      MGA_STATECHANGE( mmesa, MGA_UPLOAD_CONTEXT );
+      FLUSH_BATCH( mmesa );
       if (mmesa->hw_stencil) {
 	 mmesa->hw.stencil_enable = ( state ) ? ~0 : 0;
       }
@@ -990,38 +1017,58 @@ void mgaEmitHwStateLocked( mgaContextPtr mmesa )
       mmesa->setup.dwgctl |= (ctx->Depth.Test)
 	  ? mmesa->hw.zmode : (DC_zmode_nozcmp | DC_atype_i);
 
+#if defined(ACCEL_ROP)
       mmesa->setup.dwgctl &= DC_bop_MASK;
-      mmesa->setup.dwgctl |= (ctx->Color._LogicOpEnabled)
+      mmesa->setup.dwgctl |= (ctx->Color.ColorLogicOpEnabled)
 	  ? mmesa->hw.rop : mgarop_NoBLK[ GL_COPY & 0x0f ];
+#endif
 
       mmesa->setup.alphactrl &= AC_src_MASK & AC_dst_MASK & AC_atmode_MASK
-	 & AC_atref_MASK & AC_alphasel_MASK;
+	  & AC_atref_MASK & AC_alphasel_MASK;
       mmesa->setup.alphactrl |= 
-	 (mmesa->hw.alpha_func & mmesa->hw.alpha_func_enable) |
-	 (mmesa->hw.blend_func & mmesa->hw.blend_func_enable) |
-	 ((AC_src_one | AC_dst_zero) & ~mmesa->hw.blend_func_enable) |
-	 mmesa->hw.alpha_sel;
+	  ((mmesa->hw.alpha_func & mmesa->hw.alpha_func_enable)
+	   | ((mmesa->hw.blend_func & mmesa->hw.blend_func_enable)
+	      | ((AC_src_one | AC_dst_zero) & ~mmesa->hw.blend_func_enable))
+	   | mmesa->hw.alpha_sel
+	   | (AC_amode_alpha_channel
+	      | AC_astipple_disable 
+	      | AC_aten_disable 
+	      | AC_atmode_noacmp));
 
       memcpy( &sarea->ContextState, &mmesa->setup, sizeof(mmesa->setup));
    }
 
    if ((mmesa->dirty & MGA_UPLOAD_TEX0) && mmesa->CurrentTexObj[0]) {
+      mmesa->CurrentTexObj[0]->setup.texctl2 &= ~TMC_specen_enable;
+      mmesa->CurrentTexObj[0]->setup.texctl2 |= mmesa->hw.specen;
+
       memcpy(&sarea->TexState[0],
 	     &mmesa->CurrentTexObj[0]->setup,
 	     sizeof(sarea->TexState[0]));
    }
 
    if ((mmesa->dirty & MGA_UPLOAD_TEX1) && mmesa->CurrentTexObj[1]) {
+      mmesa->CurrentTexObj[1]->setup.texctl2 &= ~TMC_specen_enable;
+      mmesa->CurrentTexObj[1]->setup.texctl2 |= mmesa->hw.specen;
+
       memcpy(&sarea->TexState[1],
 	     &mmesa->CurrentTexObj[1]->setup,
 	     sizeof(sarea->TexState[1]));
    }
 
-   if (mmesa->dirty & (MGA_UPLOAD_TEX0 | MGA_UPLOAD_TEX1)) {
-      sarea->TexState[0].texctl2 &= ~TMC_specen_enable;
-      sarea->TexState[1].texctl2 &= ~TMC_specen_enable;
-      sarea->TexState[0].texctl2 |= mmesa->hw.specen;
-      sarea->TexState[1].texctl2 |= mmesa->hw.specen;
+   if (mmesa->dualtex_env) {
+      sarea->TexState[0].texctl2 |= TMC_dualtex_enable;
+      memcpy( &sarea->TexState[1], &sarea->TexState[0],
+	      sizeof(sarea->TexState[0]) );
+      mmesa->dirty |= MGA_UPLOAD_TEX1|MGA_UPLOAD_TEX0;
+   } else if ( (sarea->TexState[0].texctl2 & TMC_borderen_MASK) !=
+               (sarea->TexState[1].texctl2 & TMC_borderen_MASK) ) {
+      const int borderen = sarea->TexState[1].texctl2 & ~TMC_borderen_MASK;
+
+      memcpy( &sarea->TexState[1], &sarea->TexState[0],
+	      sizeof(sarea->TexState[0]) );
+      sarea->TexState[1].texctl2 |= borderen;
+      mmesa->dirty |= MGA_UPLOAD_TEX1|MGA_UPLOAD_TEX0;
    }
 
    if (mmesa->dirty & MGA_UPLOAD_PIPE) {
@@ -1032,8 +1079,19 @@ void mgaEmitHwStateLocked( mgaContextPtr mmesa )
 
    mmesa->sarea->dirty |= mmesa->dirty;
    mmesa->dirty &= MGA_UPLOAD_CLIPRECTS;
+
+   /* This is a bit of a hack but seems to be the best place to ensure
+    * that separate specular is disabled when not needed.
+    */
+   if (ctx->Texture._EnabledUnits == 0 ||
+       !ctx->Light.Enabled ||
+       ctx->Light.Model.ColorControl == GL_SINGLE_COLOR) {
+      sarea->TexState[0].texctl2 &= ~TMC_specen_enable;
+      sarea->TexState[1].texctl2 &= ~TMC_specen_enable;
+   }
 }
 
+
 /* =============================================================
  */
 
@@ -1041,21 +1099,21 @@ void mgaEmitHwStateLocked( mgaContextPtr mmesa )
 static void mgaDDValidateState( GLcontext *ctx )
 {
    mgaContextPtr mmesa = MGA_CONTEXT( ctx );
+   int new_state = mmesa->NewGLState;
+
 
    FLUSH_BATCH( mmesa );
 
-   if (mmesa->NewGLState & _NEW_TEXTURE) {
-      mgaUpdateTextureState(ctx);
+   if (mmesa->NewGLState & _MGA_NEW_RASTERSETUP) {
+      mgaChooseVertexState( ctx );
    }
 
-   if (!mmesa->Fallback) {
-      if (mmesa->NewGLState & _MGA_NEW_RASTERSETUP) {
-         mgaChooseVertexState( ctx );
-      }
+   if (mmesa->NewGLState & _MGA_NEW_RENDERSTATE) {
+      mgaChooseRenderState( ctx );
+   }
 
-      if (mmesa->NewGLState & _MGA_NEW_RENDERSTATE) {
-         mgaChooseRenderState( ctx );
-      }
+   if (new_state & _NEW_TEXTURE) {
+      mgaUpdateTextureState(ctx);
    }
 
    mmesa->NewGLState = 0;
@@ -1139,20 +1197,14 @@ void mgaInitState( mgaContextPtr mmesa )
    }
 
    mmesa->hw.blend_func = AC_src_one | AC_dst_zero;
-   mmesa->hw.blend_func_enable = 0;
-   mmesa->hw.alpha_func = AC_atmode_noacmp | MGA_FIELD( AC_atref, 0x00 );
-   mmesa->hw.alpha_func_enable = 0;
-   mmesa->hw.rop = mgarop_NoBLK[ GL_COPY & 0x0f ];
    mmesa->hw.zmode = DC_zmode_zlt | DC_atype_zi;
-   mmesa->hw.stencil = MGA_FIELD( S_sref, 0x00) | MGA_FIELD( S_smsk, 0xff ) |
-      MGA_FIELD( S_swtmsk, 0xff );
+   mmesa->hw.stencil = (0x0ff << S_smsk_SHIFT) | (0x0ff << S_swtmsk_SHIFT);
    mmesa->hw.stencilctl = SC_smode_salways | SC_sfailop_keep 
-      | SC_szfailop_keep | SC_szpassop_keep;
+       | SC_szfailop_keep | SC_szpassop_keep;
    mmesa->hw.stencil_enable = 0;
    mmesa->hw.cull = _CULL_DISABLE;
    mmesa->hw.cull_dualtex = _CULL_DISABLE;
    mmesa->hw.specen = 0;
-   mmesa->hw.alpha_sel = AC_alphasel_diffused;
 
    mmesa->setup.dwgctl = (DC_opcod_trap |
 			  DC_linear_xy |
@@ -1160,17 +1212,22 @@ void mgaInitState( mgaContextPtr mmesa )
 			  DC_arzero_disable |
 			  DC_sgnzero_disable |
 			  DC_shftzero_enable |
-			  MGA_FIELD( DC_bop, 0xC ) |
-			  MGA_FIELD( DC_trans, 0x0 ) |
+			  (0xC << DC_bop_SHIFT) |
+			  (0x0 << DC_trans_SHIFT) |
 			  DC_bltmod_bmonolef |
 			  DC_pattern_disable |
 			  DC_transc_disable |
 			  DC_clipdis_disable);
 
+
    mmesa->setup.plnwt = ~0;
-   mmesa->setup.alphactrl = (AC_amode_alpha_channel |
-			     AC_astipple_disable |
-			     AC_aten_disable);
+   mmesa->setup.alphactrl = ( AC_src_one |
+			      AC_dst_zero |
+			      AC_amode_FCOL |
+			      AC_astipple_disable |
+			      AC_aten_disable |
+			      AC_atmode_noacmp |
+			      AC_alphasel_fromtex );
 
    mmesa->setup.fogcolor = PACK_COLOR_888((GLubyte)(ctx->Fog.Color[0]*255.0F),
 					  (GLubyte)(ctx->Fog.Color[1]*255.0F),
@@ -1182,8 +1239,9 @@ void mgaInitState( mgaContextPtr mmesa )
    mmesa->setup.fcol = 0;
    mmesa->dirty |= MGA_UPLOAD_CONTEXT;
 
-   mmesa->envcolor[0] = 0;
-   mmesa->envcolor[1] = 0;
+   mmesa->envcolor = 0;
+   mmesa->blend_flags = MGA_BLEND_RGB_ZERO | MGA_BLEND_ALPHA_ZERO;
+   mmesa->dualtex_env = GL_FALSE;
 }
 
 
@@ -1200,6 +1258,7 @@ void mgaDDInitStateFuncs( GLcontext *ctx )
    ctx->Driver.DepthMask = mgaDDDepthMask;
    ctx->Driver.Fogfv = mgaDDFogfv;
    ctx->Driver.Scissor = mgaDDScissor;
+   ctx->Driver.ShadeModel = mgaDDShadeModel;
    ctx->Driver.CullFace = mgaDDCullFaceFrontFace;
    ctx->Driver.FrontFace = mgaDDCullFaceFrontFace;
    ctx->Driver.ColorMask = mgaDDColorMask;
