@@ -72,7 +72,7 @@ SOFTWARE.
  *      1, and an otherwise arbitrary ID in the low 22 bits, we can create a
  *      resource "owned" by the client.
  */
-/* $XFree86: xc/programs/Xserver/dix/resource.c,v 3.14 2003/11/17 22:20:34 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/dix/resource.c,v 3.13 2003/09/24 02:43:13 dawes Exp $ */
 
 #define NEED_EVENTS
 #include "X.h"
@@ -95,7 +95,9 @@ SOFTWARE.
 #include <assert.h>
 
 static void RebuildTable(
+#if NeedFunctionPrototypes
     int /*client*/
+#endif
 );
 
 #define SERVER_MINID 32
@@ -251,7 +253,13 @@ InitClientResources(client)
 
 
 static int
+#if NeedFunctionPrototypes
 Hash(int client, register XID id)
+#else
+Hash(client, id)
+    int client;
+    register XID id;
+#endif
 {
     id &= RESOURCE_ID_MASK;
     switch (clientTable[client].hashsize)
@@ -273,11 +281,17 @@ Hash(int client, register XID id)
 }
 
 static XID
+#if NeedFunctionPrototypes
 AvailableID(
     register int client,
     register XID id,
     register XID maxid,
     register XID goodid)
+#else
+AvailableID(client, id, maxid, goodid)
+    register int client;
+    register XID id, maxid, goodid;
+#endif
 {
     register ResourcePtr res;
 
