@@ -238,12 +238,12 @@ int i830_dma_cleanup(drm_device_t *dev)
 {
 	drm_device_dma_t *dma = dev->dma;
 
-#if __HAVE_IRQ
+#if _HAVE_DMA_IRQ
 	/* Make sure interrupts are disabled here because the uninstall ioctl
 	 * may not have been called from userspace and after dev_private
 	 * is freed, it's too late.
 	 */
-	if (dev->irq_enabled) DRM(irq_uninstall)(dev);
+	if (dev->irq) DRM(irq_uninstall)(dev);
 #endif
 
 	if (dev->dev_private) {
@@ -1546,7 +1546,7 @@ int i830_getparam( struct inode *inode, struct file *filp, unsigned int cmd,
 
 	switch( param.param ) {
 	case I830_PARAM_IRQ_ACTIVE:
-		value = dev->irq_enabled;
+		value = dev->irq ? 1 : 0;
 		break;
 	default:
 		return -EINVAL;
