@@ -28,7 +28,7 @@
  * holders shall not be used in advertising or otherwise to promote the sale,
  * use or other dealings in this Software without prior written authorization.
  */
-/* $XFree86: xc/programs/Xserver/miext/rootless/rootlessWindow.c,v 1.11 2003/11/27 01:53:39 torrey Exp $ */
+/* $XFree86: xc/programs/Xserver/miext/rootless/rootlessWindow.c,v 1.8 2003/10/25 01:05:59 torrey Exp $ */
 
 #include "rootlessCommon.h"
 #include "rootlessWindow.h"
@@ -131,15 +131,13 @@ RootlessDestroyWindow(WindowPtr pWin)
 static Bool
 RootlessGetShape(WindowPtr pWin, RegionPtr pShape)
 {
-    ScreenPtr pScreen = pWin->drawable.pScreen;
-
     if (wBoundingShape(pWin) == NULL)
         return FALSE;
 
     /* wBoundingShape is relative to *inner* origin of window.
        Translate by borderWidth to get the outside-relative position. */
 
-    REGION_NULL(pScreen, pShape);
+    REGION_INIT(pScreen, pShape, NullBox, 0);
     REGION_COPY(pScreen, pShape, wBoundingShape(pWin));
     REGION_TRANSLATE(pScreen, pShape, pWin->borderWidth, pWin->borderWidth);
 
@@ -304,7 +302,7 @@ RootlessInitializeFrame(WindowPtr pWin, RootlessWindowRec *winRec)
     winRec->borderWidth = bw;
 
 #ifdef ROOTLESS_TRACK_DAMAGE
-    REGION_NULL(pScreen, &winRec->damage);
+    REGION_INIT(pScreen, &winRec->damage, NullBox, 0);
 #endif
 }
 
@@ -611,7 +609,7 @@ RootlessResizeCopyWindow(WindowPtr pWin, DDXPointRec ptOldOrg,
     dx = ptOldOrg.x - pWin->drawable.x;
     dy = ptOldOrg.y - pWin->drawable.y;
     REGION_TRANSLATE(pScreen, prgnSrc, -dx, -dy);
-    REGION_NULL(pScreen, &rgnDst);
+    REGION_INIT(pScreen, &rgnDst, NullBox, 0);
     REGION_INTERSECT(pScreen, &rgnDst, &pWin->borderClip, prgnSrc);
 
     if (gResizeDeathCount == 1) {
@@ -630,7 +628,7 @@ RootlessResizeCopyWindow(WindowPtr pWin, DDXPointRec ptOldOrg,
 
         for (i = 0; i < gResizeDeathCount; i++) {
             REGION_INIT(pScreen, &clip, gResizeDeathBounds + 0, 1);
-            REGION_NULL(pScreen, &clipped);
+            REGION_INIT(pScreen, &clipped, NullBox, 0);
             REGION_INTERSECT(pScreen, &rgnDst, &clip, &clipped);
 
             fbCopyRegion(&gResizeDeathPix[i]->drawable,
@@ -675,7 +673,7 @@ RootlessCopyWindow(WindowPtr pWin, DDXPointRec ptOldOrg, RegionPtr prgnSrc)
     dy = ptOldOrg.y - pWin->drawable.y;
     REGION_TRANSLATE(pScreen, prgnSrc, -dx, -dy);
 
-    REGION_NULL(pScreen, &rgnDst);
+    REGION_INIT(pScreen, &rgnDst, NullBox, 0);
     REGION_INTERSECT(pScreen, &rgnDst, &pWin->borderClip, prgnSrc);
 
     extents = REGION_EXTENTS(pScreen, &rgnDst);
@@ -1042,6 +1040,7 @@ FinishFrameResize(WindowPtr pWin, Bool gravity, int oldX, int oldY,
 {
     ScreenPtr pScreen = pWin->drawable.pScreen;
     RootlessWindowRec *winRec = WINREC(pWin);
+    BoxRec box;
     int i;
 
     RootlessStopDrawing(pWin, FALSE);
@@ -1064,7 +1063,12 @@ FinishFrameResize(WindowPtr pWin, Bool gravity, int oldX, int oldY,
     /* Redraw everything. FIXME: there must be times when we don't need
        to do this. Perhaps when top-left weighting and no gravity? */
 
-    RootlessDamageRect(pWin, -newBW, -newBW, newW, newH);
+    box.x1 = 0;
+    box.y1 = 0;
+    box.x2 = winRec->width;
+    box.y2 = winRec->height;
+
+    RootlessDamageBox(pWin, &box);
 
     for (i = 0; i < 2; i++) {
         if (gResizeDeathPix[i] != NULL) {
